using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Sockets;
using System.IO;
using System.Globalization;

namespace MMClient_CS
{
    class Program
    {
        static Stanje parse(string x, string y, string d, string v)
        {
            return new Stanje(Double.Parse(x, CultureInfo.InvariantCulture), 
                Double.Parse(y, CultureInfo.InvariantCulture), 
                Double.Parse(d, CultureInfo.InvariantCulture), 
                Double.Parse(v, CultureInfo.InvariantCulture));
        }

        static void event_loop(TcpClient cli, StreamReader reader, StreamWriter writer)
        {
            try
            {
                Zival z;
                string cmd;
                string[] cmda;
                char[] delimiter = new char[] { ' ' };

                while (true)
                {
                    cmd = reader.ReadLine();
                    cmda = cmd.Split(delimiter);

                    if (cmda[0] == "ADIJO") return;
                    if (cmda[0].StartsWith("KONEC_IGRE")) continue;
                    if (cmda[0] != "ZBUDI_SE") throw new Exception("Napaka pri komunikaciji s streznikom.");
                    Koordinate j0 = new Koordinate(Double.Parse(cmda[1], CultureInfo.InvariantCulture), 
                                                    Double.Parse(cmda[2], CultureInfo.InvariantCulture));
                    Koordinate n0 = new Koordinate(Double.Parse(cmda[3], CultureInfo.InvariantCulture), 
                                                    Double.Parse(cmda[4], CultureInfo.InvariantCulture));

                    double ZeljenaSmer;
                    z = new Zival();
                    z.ZbudiSe(j0, n0, out ZeljenaSmer);
                    writer.WriteLine("BUDNA " + ZeljenaSmer.ToString());

                    while (true)
                    {
                        cmd = reader.ReadLine();
                        cmda = cmd.Split(delimiter);

                        if (cmda[0] == "ADIJO") return;
                        if (cmda[0] == "KONEC_IGRE")
                        {
                            int razlog = Int32.Parse(cmda[1]);
                            z.KonecIgre((Zival.razlog)razlog);
                            z = null;
                            break;
                        }
                        else if (cmda[0] == "QV")
                        {
                            Stanje j = parse(cmda[1], cmda[2], cmda[3], cmda[4]);
                            Stanje n = parse(cmda[5], cmda[6], cmda[7], cmda[8]);
                            double SpremembaHitrosti = 0.0, SpremembaSmeri = 0.0;
                            z.KamGres(j, n, out SpremembaSmeri, out SpremembaHitrosti);
                            writer.WriteLine("QVR " + SpremembaSmeri.ToString() + " " + SpremembaHitrosti.ToString());
//                            Console.WriteLine(String.Format(CultureInfo.InvariantCulture, 
//                                    "d={0:F5} dD={1:F5} v={2:F5} dV={3:F}", j.smer.d, SpremembaSmeri, j.smer.v, SpremembaHitrosti));
                        }
                        else
                        {
                        }
                    }
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("Napaka: " + e.ToString());
            }
        }

        static TcpClient connect(string hostname, out StreamReader reader, out StreamWriter writer)
        {
            TcpClient client = new TcpClient(hostname, 1174);
            reader = new StreamReader(client.GetStream());
            writer = new StreamWriter(client.GetStream());
            writer.AutoFlush = true;

            string server = reader.ReadLine();
            Console.WriteLine(server);

            string tip = Zival.Tip == Zival.pasma.pasmaMuca ? "MACKA" : "MIS";
            writer.WriteLine("AVE " + tip + " " + Zival.Ime);
            string succ = reader.ReadLine();
            if (succ != "PRIJAVA USPESNA") throw new Exception("Prijava na streznik neuspesna: " + succ);
            return client;
        }

        static void Main(string[] args)
        {
            string hostname;
            if (args.Length < 2) hostname = "localhost";
            else hostname = args[1];

            StreamReader reader;
            StreamWriter writer;
            TcpClient client = connect(hostname, out reader, out writer);
            event_loop(client, reader, writer);
        }
    }

    /// <summary>
    /// Pozicija zivali
    /// </summary>
    class Koordinate
    {
        public double x, y;
        public Koordinate(double _x, double _y)
        {
            x = _x;
            y = _y;
        }
    }

    /// <summary>
    /// Smer gibanja; d = kot v radianih, v = hitrost v 1.0/korak
    /// </summary>
    class Smer
    {
        public double d, v;
        public Smer(double _d, double _v)
        {
            d = _d;
            v = _v;
        }
    }

    /// <summary>
    /// Pozicija in smer zivali
    /// </summary>
    class Stanje
    {
        public Koordinate koordinate;
        public Smer smer;
        public Stanje(double x, double y, double d, double v)
        {
            koordinate = new Koordinate(x, y);
            smer = new Smer(d, v);
        }
    }
}
